/*
 * ChessCanvas.java
 *
 * Created on 1. Dezember 2007, 22:07
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

import de.gjlay.mobile.ui.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

/**
 * A Canvas that is capable of painting a chess board and chess tiles.
 * @see Board
 * @author Georg-Johann Lay
 */
abstract class ChessCanvas
        extends Canvas
{
    static final int SIZE = 21;

    private final static int C_FIELD_WHITE = 0xffffff;
    private final static int C_FIELD_BLACK = 0xb0b0b0;

    private final static int C_KING_WHITE = 0x0080ff;
    private final static int C_KING_BLACK = 0xff0000;
    private final static int C_KING_CHECK = 0xffffff;

    private final static int C_BORDER_FRAME = 0x000000;
    private final static int C_BORDER_BACK = 0xd09030;
    private final static int C_BACK = 0x000000;

    private final static int C_MOVER_WHITE = C_FIELD_WHITE;
    private final static int C_MOVER_BLACK = ((int) (C_FIELD_BLACK & 0xff)/3) * 0x020202;

    private final static int INTERVAL = 700;

    private Thread blinkThread;
    private int blink;
    protected int cursorCount;

    static final int CURSOR_MARK  = 1;
    static final int CURSOR_BLINK = 2;
    static final int CURSOR_MARKBLINK  = 3;

    boolean turn      = false; // board

    protected Board drawBoard;
    protected int drawMover;
    protected Graphics drawGraph;

    protected static Tiles chessTiles;

    // 6*3*(21*21)
    private static final String CHESS_TILES = "/res/images/chess-tiles.png";

    static MIDlet midlet;
    static
    {
        try
        {
            chessTiles = new Tiles (Image.createImage (CHESS_TILES), SIZE, SIZE);
        }
        catch (Exception e)
        {
            //e.printStackTrace ();
        }
    }

    ChessCanvas (MIDlet midlet)
    {
        this.midlet = midlet;
        blinkThread = getBlinkThread();
        blinkThread.setPriority (Thread.MIN_PRIORITY);
        blinkThread.start();
    }

    protected void setDrawContext (Graphics graph, Board board, int mover)
    {
        drawGraph = graph;
        drawBoard = board;
        drawMover = mover;
    }

    protected void drawBackground (int color)
    {
        drawGraph.setColor (color);
        drawGraph.fillRect (0, 0, getWidth(), getHeight());
    }
    /////////////////////////////////////////////////////////////
    protected void drawFrame ()
    {
        int width = 5;

        drawSquare (C_BORDER_BACK, width, true);
        drawSquare (C_BORDER_FRAME, width+1, false);
        drawSquare (C_BORDER_FRAME, 1, false);
        
        int sx = 6;
        drawGraph.drawRect (4*SIZE-sx/2-1, 8*SIZE,   sx+1, width);
        drawGraph.drawRect (4*SIZE-sx/2-1, -width-1, sx+1, width);
        
        drawGraph.drawRect (-width-1, 8*SIZE, width, width);
        drawGraph.setColor ((drawMover == Board.WHITE) ? C_MOVER_WHITE : C_MOVER_BLACK);
        drawGraph.fillRect (-width, 8*SIZE+1, width-1, width-1);

        drawGraph.setColor (turn ? C_MOVER_BLACK : C_MOVER_WHITE);
        drawGraph.fillRect (4*SIZE-sx/2, 8*SIZE+1, sx, width-1);
        
        drawGraph.setColor (turn ? C_MOVER_WHITE : C_MOVER_BLACK);
        drawGraph.fillRect (4*SIZE-sx/2, -width, sx, width-1);
    }

    protected void drawSquare (int color, int r, boolean fill)
    {
        drawGraph.setColor (color);

        if (fill)
            drawGraph.fillRect (-r, -r, 8*SIZE+2*r, 8*SIZE+2*r);
        else
            // DRAW-RECT SIZE -=1 damit = fill-rect
            drawGraph.drawRect (-r, -r, 8*SIZE+2*r -1, 8*SIZE+2*r -1);
    }

    /////////////////////////////////////////////////////////////
    protected void drawBoard ()
    {
        for (int x=0; x<8; x++)
        for (int y=0; y<8; y++)
        {
            drawGraph.setColor (((x+y) % 2 == 0) ? C_FIELD_BLACK : C_FIELD_WHITE);
            drawGraph.fillRect (SIZE*x, SIZE*(7-y), SIZE, SIZE);
        }
    }

    protected void drawPieces ()
    {
        for (int x=0; x<8; x++)
        for (int y=0; y<8; y++)
        {
            int color = drawBoard.c (x,y);
            int id = GetTileIndex (drawBoard.p(x,y), color);

            if (id < 0)
                continue;

            int yy = turn ? y : 7-y;
            int xx = turn ? 7-x : x;
            chessTiles.paint (drawGraph, id, SIZE*xx, SIZE*yy, false, 0);
        }
    }

    protected void drawCheck (int color)
    {
        if (0 == blink || !drawBoard.canMove)
            drawKingHilit (color, C_KING_CHECK);
    }

    private void drawKingHilit (int color, int theColor)
    {
        Field king = drawBoard.king[color];

        int xx = SIZE * (turn ? 7-king.x : king.x) + 10;
        int yy = SIZE * (turn ? king.y : 7-king.y) + 7;

        drawGraph.setColor (theColor);
        drawGraph.drawLine (xx, yy, xx, yy+1);
    }
    
    private static int GetTileIndex (int id, int color)
    {
        if (Board.EMPTY == id)
            return -1;

        return (id >> 1) + 6*color -1;
    }

    /////////////////////////////////////////////////////////////
    protected void drawCursor (int style, int x, int y)
    {
        int yy = SIZE * (turn ? y : 7-y);
        int xx = SIZE * (turn ? 7-x : x);

        if (style == CURSOR_MARK
                || (style == CURSOR_MARKBLINK && blink == 0))
            chessTiles.paint (drawGraph, 12 + drawMover, xx, yy, false, 0);

        if (style == CURSOR_BLINK)
            chessTiles.paint (drawGraph, (1==blink) ? 17 : 14+drawMover, xx, yy, false, 0);

        if (style == CURSOR_MARKBLINK && blink == 1)
            chessTiles.paint (drawGraph, 16, xx, yy, false, 0);
    }
    ////////////////////////////////////////////////////////////////

    private Thread getBlinkThread()
    {
        Thread thread = new Thread (new Runnable()
        {
            public void run() {
                while (true) {
                    try { Thread.sleep (50); }
                    catch (InterruptedException ex) {}

                    synchronized (this) {
                        blink = cursorCount / (INTERVAL/50);
                        if (0 == cursorCount % (INTERVAL/50))
                            repaint();
                        cursorCount = (1+cursorCount) % (2*INTERVAL/50);
        }}}});

        return thread;
    }
}
