/*
 * Move.java
 *
 * Created on 6. November 2007, 20:46
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

/**
 *
 * @author Georg-Johann Lay
 */
class Move extends java.util.Vector
{
    static final String STR_KILLS = ":";
    static final String STR_MOVES = "-";
    private static final String STR_EP    = " e.p.";
    private static final String STR_CHECK = " +";
    static final String STR_0_0   = "0-0";
    static final String STR_0_0_0 = "0-0-0";

    Field f_from;
    Field f_to;

    int color;
    boolean isCheck;
    boolean isPawnMorph = false;
    
    private int morphTo;

    private int lastPawn2x = -1;
    private int old_lastPawn2x;
    private int state;
    private int old_state;

    Move (Board board, Field f_from, Field f_to, int morphTo)
    {
        super ();

        int x0,y0, x1,y1;

        //this.board = board;
        this.morphTo = morphTo;

        this.f_from = f_from;
        this.f_to   = f_to;

        int id  = board.p (x0=f_from.x, y0=f_from.y);
        int old = board.p (x1=f_to.x,   y1=f_to.y);

        color = board.c (x0,y0);

        old_lastPawn2x    = board.lastPawn2x;
        old_state = state = board.mayCastle;

        move (board, x0, y0, x1, y1);

        int base = (color == Board.WHITE) ? 0 : 7;

        if (Board.ROOK == id)
        {
            if (y0 == base && x0 == 0)    state &= ~Board.M_0_0_0[color];
            if (y0 == base && x0 == 7)    state &= ~Board.M_0_0[color];
        }

        if (Board.KING == id)
        {
            state &= ~Board.M_0_0[color];
            state &= ~Board.M_0_0_0[color];

            if (x1 == x0+2) // 0-0
                move (board, 7, y0, 5, y0);

            if (x1 == x0-2) // 0-0-0
                move (board, 0, y0, 3, y0);
        }

        if (Board.PAWN == id)
        {
            if (x0 == x1  &&  2 == Math.abs (y0-y1))
                lastPawn2x = x0;

            // pawn takes pawn e.p.
            if (x0 != x1  &&  Board.EMPTY == old)
                remove (board, x1,y0);

            if (y1 == 7-base)
            {
                isPawnMorph = true;

                addSet1 (board, x1, y1, morphTo);
            }
        }
    }

    private int move (Board board, int x0, int y0, int x1, int y1)
        {
            int pp = board.p[x1][y1];
            addSet1 (board, x1, y1, board.p[x0][y0]);
            remove (board, x0, y0);

            return pp;
        }

    private int remove (Board board, int x, int y)
    {
        int pp = board.p[x][y];

        if (pp != Board.EMPTY)
            addSet1 (board, x, y, Board.EMPTY);

        return pp;
    }

    private void addSet1 (Board board, int x, int y, int pp)
    {
        addElement (new Set1 (board, x, y, pp));
    }

    void apply (Board board)
    {
        for (int i = 0; i < size(); i++)
                ((Set1) elementAt(i)).apply(board);

        board.lastPawn2x = lastPawn2x;
        board.mayCastle      = state;
        board.mover ^= 1;
    }

    void undo (Board board)
    {
        for (int i = size()-1; i >= 0; i--)
                ((Set1) elementAt(i)).undo(board);

        board.lastPawn2x = old_lastPawn2x;
        board.mayCastle      = old_state;
        board.mover ^= 1;
    }

    // 16 bit
    // [ 5: 0] = f_from
    // [11: 6] = f_to
    // [15:12] = morphTo
    short encode ()
    {
        int code = f_from.encode() | (f_to.encode() << 6) | (morphTo << 12);
        return (short) code;
    }

    static Move Decode (Board b, short scode)
    {
        Field from = Field.Decode (scode & 0x3f);
        Field to   = Field.Decode ((scode>>6) & 0x3f);
        int morph  = (scode>>12) & 0xf;

        return new Move (b, from, to, morph);
    }
    ////////////////////////////////////////////////////////////////////////


    class Set1
    {
        private int x, y;
        private int p_old, p_new;

        Set1 (Board b, int x, int y, int pp)
        {
            this.x = x;
            this.y = y;

            p_old = b.p[x][y];
            p_new = pp;
        }

        void apply (Board b)
        {
            b.p[x][y] = p_new;
            b.setKing (x,y);
        }

        void undo (Board b)
        {
            b.p[x][y] = p_old;
            b.setKing (x,y);
        }
    }
}

