/*
 * PlayCanvas.java
 *
 * Created on 4. November 2007, 20:51
 */

/*
 The contents of this file are subject to the
 Mozilla Public License Version 1.1 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.mozilla.org/MPL/

 Software distributed under the License is distributed on an "AS IS" basis,
 WITHOUT WARRANTY OF ANY KIND, either express or implied.
 See the License for the specific language governing rights
 and limitations under the License.

 The Initial Developer of the Original Code is
 Georg-Johann Lay (GeorgJohann@web.de).

 Portions created by the Initial Developer are
 Copyright (C) 2007 Georg-Johann Lay. All Rights Reserved.

 Alternatively, the contents of this file may be used under the terms
 of either the GNU General Public License Version 3 or later (the "GPL"),
 or the GNU Lesser General Public License Version 3 or later (the "LGPL"),
 in which case the provisions of the GPL or the LGPL are applicable instead
 of those above.

 If you wish to allow use of your version of this file only
 under the terms of either the GPL or the LGPL, and not to allow others to
 use your version of this file under the terms of the MPL, indicate your
 decision by deleting the provisions above and replace them with the notice
 and other provisions required by the LGPL or the GPL. If you do not delete
 the provisions above, a recipient may use your version of this file under
 the terms of any one of the MPL, the GPL or the LGPL.
*/

package de.gjlay.chess;

import de.gjlay.mobile.ui.*;
import javax.microedition.lcdui.*;
import javax.microedition.midlet.MIDlet;

/**
 *
 * @author Georg-Johann Lay
 */
final class PlayCanvas
    extends ChessCanvas
    implements SelectListener
{
    private static final String LOVER_ID = "last";
    
    private static final int SEL_FROM  = 0;
    private static final int SEL_TO    = 1;
    private static final int SEL_MORPH = 2;

    private static final int ID_TURN  = 1;
    private static final int ID_EXIT  = 2;
    private static final int ID_RESET = 3;
    private static final int ID_UNDO  = 4;
    private static final int ID_REDO  = 5;

    // selectionDone
    // messageReturned
    private static PlayCanvas instance;

    private SelectorLabel[] mainLabels;

    private Board b;

    private int px, py;

    /** Will be one of {@link #SEL_FROM}, {@link #SEL_TO}, {@link #SEL_MORPH}.
     */
    private int stage = SEL_FROM;
    private int mover;

    /** {@link #move} will be from this {@link Field}. */
    private Field f_from;

    /** {@link #move} will be to that {@link Field}. */
    private Field f_to;

    private Moves moves = null;

    private Move move;
    private int morphIndex = 0;

    PlayCanvas (MIDlet midlet)
    {
        super (midlet);
        
        mainLabels = new SelectorLabel[]
        {
            new SelectorLabel (ID_UNDO,  "<*> Undo Move"),
            new SelectorLabel (ID_REDO,  "<#> Redo Move"),
            new SelectorLabel (ID_TURN,  "<1> Turn Board"),
            new SelectorLabel (ID_RESET, "Reset Board"),
            new SelectorLabel (ID_EXIT,  "Exit")
        };
    }

    static PlayCanvas GetInstance (MIDlet midlet)
    {
        return instance = new PlayCanvas (midlet);
    }

    private void initCursor()
    {
        Move m;

        mover = b.mover;

        if (null != (m = undo (false)))
        {
            m = redo();
            px = m.f_to.x;
            py = m.f_to.y;
        }
        else if (null != (m = redo()))
        {
            m = undo (false);
            px = m.f_from.x;
            py = m.f_from.y;
        }
        else
        {
            px = 4;
            py = 0;
        }
    }

    private Move undo (boolean doSave)
    {
        Move m = b.undo();

        if (doSave)
            b.save (LOVER_ID);

        return m;
    }

    private Move redo()
    {
        Move m = b.redo();
        b.save (LOVER_ID);
        return m;
    }

    private void apply (Move m, boolean doSave)
    {
        b.apply (m);
        if (doSave)
            b.save (LOVER_ID);
    }

    public void keyPressed (int code)
    {
        cursorCount = 0;
        boolean done = true;

        switch (code)
        {
            default:
                done = false;
                break;

            case KEY_STAR:
            case KEY_POUND:
                handleUndoRedo (code);
                break;

            case KEY_NUM1:
                turn = !turn;
                break;

            case KEY_NUM9:
                showMainMenu();
                return;
        }

        if (done)
        {
            repaint();
            return;
        }

        int game = getGameAction (code);

        if (stage == SEL_MORPH)
            handleMorph (game);
        else
            handlePlay (game);

        repaint();
    }

    private void handlePlay (int game)
    {
        boolean done = true;

        int dd = turn ? -1 : 1;

        switch (game)
        {
            default:               return;
            case FIRE:             done = false;            break;
            case UP:               py += dd;                break;
            case DOWN:             py -= dd;                break;
            case RIGHT:            px += dd;                break;
            case LEFT:             px -= dd;                break;
        }

        if (done)
        {
            px = Math.min (7, Math.max(0, px));
            py = Math.min (7, Math.max(0, py));
            return;
        }

        // FIRE
        switch (stage)
        {
            case SEL_FROM:

                moves = b.getMoves (px, py);
                if (moves.isEmpty ())
                {
                    moves = null;
                    return;
                }

                stage = SEL_TO;
                f_from = Field.Get (px, py);
                break;

            case SEL_TO:
                f_to = Field.Get (px, py);

                if (f_to.equals (f_from))
                {
                    stage = SEL_FROM;
                    break;
                }

                Moves movesF = moves.filter (f_from, f_to);

                if (movesF.size() == 0)
                    break;
                
                moves = movesF;
                move = (Move) moves.firstElement();

                if (moves.size() == 1)
                {
                    apply (move, true);
                    stage = SEL_FROM;
                    moves = null;
                    mover = b.mover;
                    break;
                }

                if (moves.size() > 1)
                {
                    apply (move, false);
                    morphIndex = 0;
                    stage = SEL_MORPH;
                }
                break;
        }
    }

    private void handleMorph (int game)
    {
        switch (game)
        {
            case UP:
            case RIGHT:
                morphIndex += 2;
            case DOWN:
            case LEFT:
                morphIndex --;

                move = (Move) moves.elementAt (morphIndex = (4+morphIndex) % 4);
                undo (false);
                apply (move, false);
                break;

            case FIRE:
                undo (false);
                apply (move, true);
                stage = SEL_FROM;
                mover = b.mover;
                break;
        }
    }

    private void handleUndoRedo (int code)
    {
        Field f = null;
        Move  m;

        if (stage == SEL_MORPH)
        {
            m = b.removeLastMove();
            f = m.f_from;
        }
        else
        {
            if (KEY_POUND == code)
                if ((m = redo ()) != null)
                    f = m.f_to;

            if (KEY_STAR == code)
                if ((m = undo (true)) != null)
                    f = m.f_from;
        }

        stage = SEL_FROM;
        if (f != null)
        {
            px = f.x;
            py = f.y;
        }

        mover = b.mover;
    }

    public void paint (Graphics g)
    {
        setDrawContext (g, b, mover);

        g.translate(- g.getTranslateX(), - g.getTranslateY());

        drawBackground (0x006000);

        g.translate (getWidth()/2-4*SIZE, getHeight()/2-4*SIZE);

        drawFrame ();
        drawBoard ();

        if (b.canMove || stage == SEL_MORPH)
            drawCursors ();
        drawPieces ();
    }
    
    protected void drawPieces ()
    {
        super.drawPieces ();
        // Schach ?
        if (b.isCheck())
            drawCheck (b.mover);
    }

    private void drawCursors ()
    {
        boolean high = (stage == SEL_TO
                    && px == f_from.x && py == f_from.y)
                    || stage == SEL_MORPH;
        drawCursor (high ? CURSOR_MARKBLINK : CURSOR_BLINK, px, py);

        if (!high && stage == SEL_TO)
            drawCursor (CURSOR_MARK, f_from.x, f_from.y);
    }

    static void Show (boolean newGame)
    {
        if (newGame)
            instance.startGame();
        else
            instance.resumeGame();
    }

    private void startGame()
    {
        b = new Board();
        b.load (LOVER_ID);

        initCursor();
        resumeGame();
    }

    private void resumeGame()
    {
        //Drawer.canvas = this;
        setFullScreenMode (true);
        Display.getDisplay (midlet).setCurrent (this);

        repaint();
    }

    void resetBoard()
    {
        b.reset();
        b.save (LOVER_ID);

        mover = b.mover;
        px = 4; //b.king[Board.WHITE].x;
        py = 0; //b.king[Board.WHITE].y;

        stage = SEL_FROM;
    }

    private void showMainMenu()
    {
        Selector.Show (this, 0, "Chess Board", mainLabels);
    }

    public void selectionDone (int id, int tag)
    {
        switch (id)
        {
            default:
                return;

            case Selector.BACK:
                break;

            case ID_UNDO:
                handleUndoRedo (KEY_STAR);
                break;
                
            case ID_REDO:
                handleUndoRedo (KEY_POUND);
                break;
                
            case ID_TURN:
                turn = !turn;
                break;
                
            case ID_EXIT:
                ChessLET.Exit();
                return;

            case ID_RESET:
                resetBoard ();
                break;
        }

        resumeGame();
    }
}
